package org.cis1200;

import java.util.List;
import java.util.LinkedList;

public class ChatterBot {

    /** The MarkovChain you'll be using to generate posts */
    private final MarkovChain mc;

    /**
     * @param trainingData - cleaned data from which to construct the ChatterBot
     *                     MarkovModel
     */
    public ChatterBot(List<List<String>> trainingData) {
        mc = new MarkovChain(trainingData);
    }

    /**
     * Generates a single, well-formatted post by following a random walk
     * through the MarkovModel. The path is given by the choices made by
     * {@code ng}.
     * <p>
     * A generated post consists of the sequence of tokens produced by
     * the random walk, with spaces added after every token that
     * is not itself followed by a punctuation token.
     * <p>
     * For example, a walk through the MarkovChain of the illustrative example
     * (see {@link MarkovChain}) might produce the following sequence of tokens:
     * 
     * <pre>
     *     "a" "banana" "!" "and" "a" "chair"
     * </pre>
     * 
     * They would be concatenated with spaces (except before the "!" and at the end)
     * to
     * produce the post: {@code "a banana! and a chair"}.
     * <p>
     * <b>NOTE:</b> The {@link StringBuilder} class provides a useful way to append
     * {@code String}s and the method {@link String#matches(String)} can be used
     * with
     * {@link PostParser#PUNCTUATION_TOKEN} to determine whether a token is
     * punctuation.
     *
     * @param ng a source of numbers for walking through the ChatterBot's
     *           MarkovChain
     * @return the post generated by following that walk
     */
    public String generatePost(NumberGenerator ng) {
        return null; // TODO: Complete this method
    }

    /**
     * Generates a random post. This function is implemented for you.
     * 
     * @return a randomly generated post
     */
    public String generatePost() {
        return generatePost(new RandomNumberGenerator());
    }

    /**
     * Generates a list containing a specified number of randomly generated posts.
     *
     * This function is implemented for you.
     *
     * @param numPosts number of posts to generate
     * @return a list of random posts of length {@code numPosts}
     */
    public List<String> generateRandomPosts(int numPosts) {
        List<String> posts = new LinkedList<>();
        while (numPosts > 0) {
            posts.add(generatePost());
            numPosts--;
        }
        return posts;
    }

}
