package org.cis1200;

import java.io.*;
import java.util.List;

/**
 * This is the class where everything you've worked on comes together!
 * You can see that we've provided a path to a CSV file of posts and the
 * column from which they can be extracted. When run as an application, this
 * program builds a Markov Chain from the training data in the CSV file,
 * generates 10 random posts, and prints them to the terminal.
 * <p>
 * There is nothing for you to do in this file -- it already contains the
 * code necessary to put your ChatterBot to work.
 * <p>
 * When run on the data from the illustrative_example.csv, we get output that
 * looks like:
 * 
 * <pre>
 * a table and a banana! and a chair
 * a table and a banana! and a banana! and a table and a chair
 * a chair
 * a chair
 * a banana! and a table and a banana! and a banana! and a banana! and a banana! and a banana?
 * a table and a banana?
 * a banana?
 * a table and a chair
 * a chair
 * a table and a banana! and a table and a chair*
 * </pre>
 *
 * <p>
 * When run on more realistic data from factretreiver_posts.csv, we get
 * witticisms like
 * * the following (among much junkier random outputs):
 * *
 * 
 * <pre>
 *  * Octopuses can have belly buttons.
 *  * Sometimes a seed pods look like.
 *  * Tigers cannot breathe through their mouth.
 *  * The average of expensive material is called Chicken Feed.
 *  * Dogs are more efficient fliers than about his followers!
 *  * Humans can remember ditching school
 * </pre>
 */
public class ChatterBotMain {

    /**
     * This is a path to the CSV file containing the posts. The main method
     * below uses the posts in this file when calling ChatterBot. If you want
     * to run the ChatterBot on the other files we provide, change this path to
     * a different file. (You may need to adjust the POST_COLUMN too.)
     */
    static final String PATH_TO_POSTS = "files/illustrative_example.csv";
    /** Column in the PATH_TO_POSTS CSV file to read posts from */
    static final int POST_COLUMN = 2;
    /** File to store generated posts */
    static final String PATH_TO_OUTPUT_POSTS = "files/generated_posts.txt";

    /**
     * Prints ten generated posts to the console so that you can see how your bot is
     * performing!
     */
    public static void main(String[] args) {

        // Obtain a Reader for processing the CSV file
        BufferedReader csvReader = FileUtilities.fileToReader(PATH_TO_POSTS);

        // Extract all the CSV fields at the given POST_COLUMN
        List<String> rawPosts = CSV.csvFieldsAtColumn(csvReader, POST_COLUMN);

        // Parse the raw posts into training data
        List<List<String>> trainingData = PostParser.rawPostsToTrainingData(rawPosts);

        // Construct a trained ChatterBot
        ChatterBot cb = new ChatterBot(trainingData);

        // Uncomment the line below to see the MarkovChain produced from the given
        // training data
        // System.out.println(cb.mc.toString());

        // Generate 10 random posts and print them to the terminal
        List<String> posts = cb.generateRandomPosts(10);
        for (String post : posts) {
            System.out.println(post);
        }

        // Write the generated posts to the given file.
        // Make sure that PATH_TO_OUTPUT_POSTS is set properly.
        FileUtilities.writeStringsToFile(posts, PATH_TO_OUTPUT_POSTS, false);
    }

}
